use super::Element;
use super::Object;
use super::Source;

 type PriorityCookie = u8;

#[derive(Debug)]
 struct Document<'s> {
     source: &'s str,
     zeroth_section: Option<Section<'s>>,
     children: Vec<Heading<'s>>,
}

#[derive(Debug)]
 struct Heading<'s> {
     source: &'s str,
     stars: usize,
     todo_keyword: Option<(TodoKeywordType, &'s str)>,
     priority_cookie: Option<PriorityCookie>,
     title: Vec<Object<'s>>,
     tags: Vec<&'s str>,
     children: Vec<DocumentElement<'s>>,
     is_comment: bool,
     is_archived: bool,
}

#[derive(Debug)]
 struct Section<'s> {
     source: &'s str,
     children: Vec<Element<'s>>,
}

#[derive(Debug)]
 enum DocumentElement<'s> {
    Heading(Heading<'s>),
    Section(Section<'s>),
}

#[derive(Debug)]
 enum TodoKeywordType {
    Todo,
    Done,
}

impl<'s> Source<'s> for Document<'s> {
    fn get_source(&'s self) -> &'s str {
        self.source
    }
}

impl<'s> Source<'s> for DocumentElement<'s> {
    fn get_source(&'s self) -> &'s str {
        match self {
            DocumentElement::Heading(obj) => obj.source,
            DocumentElement::Section(obj) => obj.source,
        }
    }
}

impl<'s> Source<'s> for Section<'s> {
    fn get_source(&'s self) -> &'s str {
        self.source
    }
}

impl<'s> Source<'s> for Heading<'s> {
    fn get_source(&'s self) -> &'s str {
        self.source
    }
}
