use super::element::Element;
use super::lesser_element::TableCell;
use super::Keyword;
use super::Object;
use super::Source;

#[derive(Debug)]
 struct PlainList<'s> {
     source: &'s str,
     children: Vec<PlainListItem<'s>>,
}

#[derive(Debug)]
 struct PlainListItem<'s> {
     source: &'s str,
     indentation: usize,
     bullet: &'s str,
     tag: Vec<Object<'s>>,
     children: Vec<Element<'s>>,
}

#[derive(Debug)]
 struct GreaterBlock<'s> {
     source: &'s str,
     name: &'s str,
     parameters: Option<&'s str>,
     children: Vec<Element<'s>>,
}

#[derive(Debug)]
 struct DynamicBlock<'s> {
     source: &'s str,
     name: &'s str,
     parameters: Option<&'s str>,
     children: Vec<Element<'s>>,
}

#[derive(Debug)]
 struct FootnoteDefinition<'s> {
     source: &'s str,
     label: &'s str,
     children: Vec<Element<'s>>,
}

#[derive(Debug)]
 struct Drawer<'s> {
     source: &'s str,
     name: &'s str,
     children: Vec<Element<'s>>,
}

#[derive(Debug)]
 struct PropertyDrawer<'s> {
     source: &'s str,
     children: Vec<NodeProperty<'s>>,
}

#[derive(Debug)]
 struct NodeProperty<'s> {
     source: &'s str,
     value: Option<&'s str>,
}

#[derive(Debug)]
 struct Table<'s> {
     source: &'s str,
     formulas: Vec<Keyword<'s>>,
     children: Vec<TableRow<'s>>,
}

#[derive(Debug)]
 struct TableRow<'s> {
     source: &'s str,
     children: Vec<TableCell<'s>>,
}

impl<'s> Source<'s> for PlainList<'s> {
    fn get_source(&'s self) -> &'s str {
        self.source
    }
}

impl<'s> Source<'s> for PlainListItem<'s> {
    fn get_source(&'s self) -> &'s str {
        self.source
    }
}

impl<'s> Source<'s> for GreaterBlock<'s> {
    fn get_source(&'s self) -> &'s str {
        self.source
    }
}

impl<'s> Source<'s> for DynamicBlock<'s> {
    fn get_source(&'s self) -> &'s str {
        self.source
    }
}

impl<'s> Source<'s> for FootnoteDefinition<'s> {
    fn get_source(&'s self) -> &'s str {
        self.source
    }
}

impl<'s> Source<'s> for Drawer<'s> {
    fn get_source(&'s self) -> &'s str {
        self.source
    }
}

impl<'s> Source<'s> for PropertyDrawer<'s> {
    fn get_source(&'s self) -> &'s str {
        self.source
    }
}

impl<'s> Source<'s> for NodeProperty<'s> {
    fn get_source(&'s self) -> &'s str {
        self.source
    }
}

impl<'s> Source<'s> for Table<'s> {
    fn get_source(&'s self) -> &'s str {
        self.source
    }
}

impl<'s> Source<'s> for TableRow<'s> {
    fn get_source(&'s self) -> &'s str {
        self.source
    }
}
