use super::object::Object;
use super::PlainText;
use super::Source;

#[derive(Debug)]
 struct Paragraph<'s> {
     source: &'s str,
     children: Vec<Object<'s>>,
}

#[derive(Debug)]
 struct Comment<'s> {
     source: &'s str,
}

#[derive(Debug)]
 struct TableCell<'s> {
     source: &'s str,
     children: Vec<Object<'s>>,
}

#[derive(Debug)]
 struct VerseBlock<'s> {
     source: &'s str,
     name: &'s str,
     data: Option<&'s str>,
     children: Vec<Object<'s>>,
}

#[derive(Debug)]
 struct CommentBlock<'s> {
     source: &'s str,
     name: &'s str,
     data: Option<&'s str>,
     contents: &'s str,
}

#[derive(Debug)]
 struct ExampleBlock<'s> {
     source: &'s str,
     name: &'s str,
     data: Option<&'s str>,
     contents: &'s str,
}

#[derive(Debug)]
 struct ExportBlock<'s> {
     source: &'s str,
     name: &'s str,
     data: Option<&'s str>,
     contents: &'s str,
}

#[derive(Debug)]
 struct SrcBlock<'s> {
     source: &'s str,
     name: &'s str,
     data: Option<&'s str>,
     contents: &'s str,
}

#[derive(Debug)]
 struct Clock<'s> {
     source: &'s str,
}

#[derive(Debug)]
 struct DiarySexp<'s> {
     source: &'s str,
}

#[derive(Debug)]
 struct Planning<'s> {
     source: &'s str,
}

#[derive(Debug)]
 struct FixedWidthArea<'s> {
     source: &'s str,
}

#[derive(Debug)]
 struct HorizontalRule<'s> {
     source: &'s str,
}

#[derive(Debug)]
 struct Keyword<'s> {
     source: &'s str,
     key: &'s str,
     value: &'s str,
}

#[derive(Debug)]
 struct LatexEnvironment<'s> {
     source: &'s str,
}

impl<'s> Paragraph<'s> {
     fn of_text(input: &'s str) -> Self {
        let mut objects = Vec::with_capacity(1);
        objects.push(Object::PlainText(PlainText { source: input }));
        Paragraph {
            source: input,
            children: objects,
        }
    }
}

impl<'s> Source<'s> for Paragraph<'s> {
    fn get_source(&'s self) -> &'s str {
        self.source
    }
}

impl<'s> Source<'s> for TableCell<'s> {
    fn get_source(&'s self) -> &'s str {
        self.source
    }
}

impl<'s> Source<'s> for Comment<'s> {
    fn get_source(&'s self) -> &'s str {
        self.source
    }
}

impl<'s> Source<'s> for VerseBlock<'s> {
    fn get_source(&'s self) -> &'s str {
        self.source
    }
}
impl<'s> Source<'s> for CommentBlock<'s> {
    fn get_source(&'s self) -> &'s str {
        self.source
    }
}
impl<'s> Source<'s> for ExampleBlock<'s> {
    fn get_source(&'s self) -> &'s str {
        self.source
    }
}
impl<'s> Source<'s> for ExportBlock<'s> {
    fn get_source(&'s self) -> &'s str {
        self.source
    }
}
impl<'s> Source<'s> for SrcBlock<'s> {
    fn get_source(&'s self) -> &'s str {
        self.source
    }
}

impl<'s> Source<'s> for Clock<'s> {
    fn get_source(&'s self) -> &'s str {
        self.source
    }
}

impl<'s> Source<'s> for DiarySexp<'s> {
    fn get_source(&'s self) -> &'s str {
        self.source
    }
}

impl<'s> Source<'s> for Planning<'s> {
    fn get_source(&'s self) -> &'s str {
        self.source
    }
}

impl<'s> Source<'s> for FixedWidthArea<'s> {
    fn get_source(&'s self) -> &'s str {
        self.source
    }
}

impl<'s> Source<'s> for HorizontalRule<'s> {
    fn get_source(&'s self) -> &'s str {
        self.source
    }
}

impl<'s> Source<'s> for Keyword<'s> {
    fn get_source(&'s self) -> &'s str {
        self.source
    }
}

impl<'s> Source<'s> for LatexEnvironment<'s> {
    fn get_source(&'s self) -> &'s str {
        self.source
    }
}
